@@ -7,7 +7,6 @@
 
 #include "background.h"
 #include "background_debug.h"
-#include "utils.h"
 #include "waylandintegration.h"
 
 #include <QDBusConnection>
@@ -35,7 +34,7 @@ BackgroundPortal::BackgroundPortal(QObject *parent, QDBusContext *context)
     : QDBusAbstractAdaptor(parent)
     , m_context(context)
 {
-    connect(WaylandIntegration::waylandIntegration(), &WaylandIntegration::WaylandIntegration::plasmaWindowManagementInitialized, this, [=]() {
+    connect(WaylandIntegration::waylandIntegration(), &WaylandIntegration::WaylandIntegration::plasmaWindowManagementInitialized, this, [this]() {
         connect(WaylandIntegration::plasmaWindowManagement(),
                 &KWayland::Client::PlasmaWindowManagement::windowCreated,
                 this,
@@ -44,7 +43,7 @@ BackgroundPortal::BackgroundPortal(QObject *parent, QDBusContext *context)
                 });
 
         m_windows = WaylandIntegration::plasmaWindowManagement()->windows();
-        for (KWayland::Client::PlasmaWindow *window : qAsConst(m_windows)) {
+        for (KWayland::Client::PlasmaWindow *window : std::as_const(m_windows)) {
             addWindow(window);
         }
     });
@@ -101,57 +100,78 @@ uint BackgroundPortal::NotifyBackground(const QDBusObjectPath &handle, const QSt
     }
 
     KNotification *notify = new KNotification(QStringLiteral("notification"), KNotification::Persistent | KNotification::DefaultEvent, this);
-    notify->setTitle(i18n("Background activity"));
-    notify->setText(i18n("%1 is running in the background.", appName));
-    notify->setActions({i18n("Find out more")});
+    notify->setTitle(i18n("Background Activity"));
+    notify->setText(
+        i18nc("@info %1 is the name of an application",
+              "%1 wants to remain running when it has no visible windows. If you forbid this, the application will quit when its last window is closed.",
+              appName));
     notify->setProperty("activated", false);
 
     message.setDelayedReply(true);
 
-    connect(notify, QOverload<uint>::of(&KNotification::activated), this, [=](uint action) {
-        if (action != 1) {
-            return;
-        }
-        notify->setProperty("activated", true);
+    auto allowAction = notify->addAction(i18nc("@action:button Allow the app to keep running with no open windows", "Allow"));
 
-        const QString title = i18n("%1 is running in the background", appName);
-        const QString text = i18n(
-            "This might be for a legitimate reason, but the application has not provided one."
-            "\n\nNote that forcing an application to quit might cause data loss.");
-        QMessageBox *messageBox = new QMessageBox(QMessageBox::Question, title, text);
-        messageBox->addButton(i18n("Force quit"), QMessageBox::RejectRole);
-        messageBox->addButton(i18n("Allow"), QMessageBox::AcceptRole);
-
-        messageBox->show();
-
-        connect(messageBox, &QMessageBox::accepted, this, [message, messageBox]() {
-            const QVariantMap map = {{QStringLiteral("result"), static_cast<uint>(BackgroundPortal::Allow)}};
-            QDBusMessage reply = message.createReply({static_cast<uint>(0), map});
-            if (!QDBusConnection::sessionBus().send(reply)) {
-                qCWarning(XdgDesktopPortalKdeBackground) << "Failed to send response";
-            }
-            messageBox->deleteLater();
-        });
-        connect(messageBox, &QMessageBox::rejected, this, [message, messageBox]() {
-            const QVariantMap map = {{QStringLiteral("result"), static_cast<uint>(BackgroundPortal::Forbid)}};
-            QDBusMessage reply = message.createReply({static_cast<uint>(0), map});
-            if (!QDBusConnection::sessionBus().send(reply)) {
-                qCWarning(XdgDesktopPortalKdeBackground) << "Failed to send response";
-            }
-            messageBox->deleteLater();
-        });
-    });
-    connect(notify, &KNotification::closed, this, [=]() {
-        if (notify->property("activated").toBool()) {
-            return;
-        }
-
-        QVariantMap map;
-        map.insert(QStringLiteral("result"), static_cast<uint>(BackgroundPortal::AllowOnce));
+    auto allow = [message]() {
+        const QVariantMap map = {{QStringLiteral("result"), static_cast<uint>(BackgroundPortal::Allow)}};
         QDBusMessage reply = message.createReply({static_cast<uint>(0), map});
         if (!QDBusConnection::sessionBus().send(reply)) {
             qCWarning(XdgDesktopPortalKdeBackground) << "Failed to send response";
         }
+    };
+
+    connect(allowAction, &KNotificationAction::activated, this, [allow, notify] {
+        allow();
+        notify->setProperty("activated", true);
+    });
+
+    auto forbidAction = notify->addAction(i18nc("@action:button Don't allow the app to keep running without any open windows", "Forbid"));
+
+    auto forbid = [message]() {
+        const QVariantMap map = {{QStringLiteral("result"), static_cast<uint>(BackgroundPortal::Forbid)}};
+        QDBusMessage reply = message.createReply({static_cast<uint>(0), map});
+        if (!QDBusConnection::sessionBus().send(reply)) {
+            qCWarning(XdgDesktopPortalKdeBackground) << "Failed to send response";
+        }
+    };
+
+    connect(forbidAction, &KNotificationAction::activated, this, [this, appName, allow, forbid, notify] {
+        const QString title =
+            i18nc("@title title of a dialog to confirm whether to allow an app to remain running with no visible windows", "Background App Usage");
+        const QString text = i18nc("%1 is the name of an application",
+                                   "Note that this will force %1 to quit when its last window is closed. This could cause data loss if the application has "
+                                   "any unsaved changes when it happens.",
+                                   appName);
+        auto messageBox = new QMessageBox(QMessageBox::Question, title, text);
+        messageBox->addButton(i18nc("@action:button Allow the app to keep running with no open windows", "Allow"), QMessageBox::AcceptRole);
+        messageBox->addButton(i18nc("@action:button Don't allow the app to keep running without any open windows", "Forbid Anyway"), QMessageBox::RejectRole);
+        messageBox->show();
+
+        connect(messageBox, &QMessageBox::accepted, this, [messageBox, allow]() {
+            allow();
+            messageBox->deleteLater();
+        });
+        connect(messageBox, &QMessageBox::rejected, this, [messageBox, forbid]() {
+            forbid();
+            messageBox->deleteLater();
+        });
+
+        notify->setProperty("activated", true);
+    });
+
+    auto allowOnce = [message]() {
+        const QVariantMap map = {{QStringLiteral("result"), static_cast<uint>(BackgroundPortal::AllowOnce)}};
+        QDBusMessage reply = message.createReply({static_cast<uint>(0), map});
+        if (!QDBusConnection::sessionBus().send(reply)) {
+            qCWarning(XdgDesktopPortalKdeBackground) << "Failed to send response";
+        }
+    };
+
+    connect(notify, &KNotification::closed, this, [notify, allowOnce]() {
+        if (notify->property("activated").toBool()) {
+            return;
+        }
+
+        allowOnce();
     });
 
     notify->sendEvent();
