@@ -24,6 +24,7 @@
 #include <QDBusArgument>
 #include <QDBusMetaType>
 #include <QDataStream>
+#include <QGuiApplication>
 #include <QIODevice>
 
 struct RestoreData {
@@ -191,7 +192,7 @@ uint ScreenCastPortal::Start(const QDBusObjectPath &handle,
         return 2;
     }
 
-    if (WaylandIntegration::screens().isEmpty()) {
+    if (QGuiApplication::screens().isEmpty()) {
         qCWarning(XdgDesktopPortalKdeScreenCast) << "Failed to show dialog as there is no screen to select";
         return 2;
     }
@@ -199,7 +200,8 @@ uint ScreenCastPortal::Start(const QDBusObjectPath &handle,
     const PersistMode persist = session->persistMode();
     bool valid = false;
     QList<Output> selectedOutputs;
-    QVector<QMap<int, QVariant>> selectedWindows;
+    QList<QMap<int, QVariant>> selectedWindows;
+    QRect selectedRegion;
     if (persist != NoPersist && session->restoreData().isValid()) {
         const RestoreData restoreData = qdbus_cast<RestoreData>(session->restoreData().value<QDBusArgument>());
         if (restoreData.session == QLatin1String("KDE") && restoreData.version == RestoreData::currentRestoreDataVersion()) {
@@ -247,6 +249,7 @@ uint ScreenCastPortal::Start(const QDBusObjectPath &handle,
             allowRestore = screenDialog->allowRestore();
             selectedOutputs = screenDialog->selectedOutputs();
             selectedWindows = screenDialog->selectedWindows();
+            selectedRegion = screenDialog->selectedRegion();
         }
     }
 
@@ -255,17 +258,20 @@ uint ScreenCastPortal::Start(const QDBusObjectPath &handle,
         QStringList windows;
         WaylandIntegration::Streams streams;
         Screencasting::CursorMode cursorMode = Screencasting::CursorMode(session->cursorMode());
-        for (const auto &output : qAsConst(selectedOutputs)) {
+        for (const auto &output : std::as_const(selectedOutputs)) {
             WaylandIntegration::Stream stream;
             switch (output.outputType()) {
-            case WaylandIntegration::WaylandOutput::Workspace:
+            case Output::Region:
+                stream = WaylandIntegration::startStreamingRegion(selectedRegion, cursorMode);
+                break;
+            case Output::Workspace:
                 stream = WaylandIntegration::startStreamingWorkspace(cursorMode);
                 break;
-            case WaylandIntegration::WaylandOutput::Virtual:
+            case Output::Virtual:
                 stream = WaylandIntegration::startStreamingVirtual(output.uniqueId(), {1920, 1080}, cursorMode);
                 break;
             default:
-                stream = WaylandIntegration::startStreamingOutput(output.waylandOutputName(), cursorMode);
+                stream = WaylandIntegration::startStreamingOutput(output.screen(), cursorMode);
                 break;
             }
 
@@ -279,7 +285,7 @@ uint ScreenCastPortal::Start(const QDBusObjectPath &handle,
             }
             streams << stream;
         }
-        for (const auto &win : qAsConst(selectedWindows)) {
+        for (const auto &win : std::as_const(selectedWindows)) {
             WaylandIntegration::Stream stream = WaylandIntegration::startStreamingWindow(win, cursorMode);
             if (!stream.isValid()) {
                 qCWarning(XdgDesktopPortalKdeScreenCast) << "Invalid window!" << win;
