@@ -8,15 +8,20 @@
 
 #include "screenchooserdialog.h"
 #include "utils.h"
+#include "waylandintegration.h"
+
+#include "region-select/SelectionEditor.h"
 
 #include <KLocalizedString>
 #include <KWayland/Client/plasmawindowmanagement.h>
 #include <KWayland/Client/plasmawindowmodel.h>
 
 #include <QCoreApplication>
+#include <QScreen>
 #include <QSettings>
 #include <QSortFilterProxyModel>
 #include <QStandardPaths>
+#include <QTimer>
 #include <QWindow>
 
 class FilteredWindowModel : public QSortFilterProxyModel
@@ -72,9 +77,9 @@ public:
         return true;
     }
 
-    QVector<QMap<int, QVariant>> selectedWindows() const
+    QList<QMap<int, QVariant>> selectedWindows() const
     {
-        QVector<QMap<int, QVariant>> ret;
+        QList<QMap<int, QVariant>> ret;
         ret.reserve(m_selected.size());
         for (const auto &index : m_selected) {
             if (index.isValid())
@@ -141,7 +146,8 @@ ScreenChooserDialog::ScreenChooserDialog(const QString &appName, bool multiple,
 
     int numberOfMonitors = 0;
     if (types & ScreenCastPortal::Monitor) {
-        auto model = new OutputsModel(OutputsModel::Options(OutputsModel::WorkspaceIncluded | OutputsModel::VirtualIncluded), this);
+        auto model =
+            new OutputsModel(OutputsModel::Options(OutputsModel::WorkspaceIncluded | OutputsModel::VirtualIncluded | OutputsModel::RegionIncluded), this);
         props.insert("outputsModel", QVariant::fromValue<QObject *>(model));
         numberOfMonitors += model->rowCount(QModelIndex());
         connect(this, &ScreenChooserDialog::clearSelection, model, &OutputsModel::clearSelection);
@@ -236,7 +242,7 @@ QList<Output> ScreenChooserDialog::selectedOutputs() const
     return model->selectedOutputs();
 }
 
-QVector<QMap<int, QVariant>> ScreenChooserDialog::selectedWindows() const
+QList<QMap<int, QVariant>> ScreenChooserDialog::selectedWindows() const
 {
     FilteredWindowModel *model = dynamic_cast<FilteredWindowModel *>(m_theDialog->property("windowsModel").value<QObject *>());
     if (!model) {
@@ -245,9 +251,42 @@ QVector<QMap<int, QVariant>> ScreenChooserDialog::selectedWindows() const
     return model->selectedWindows();
 }
 
+QRect ScreenChooserDialog::selectedRegion() const
+{
+    return m_region;
+}
+
+void ScreenChooserDialog::setRegion(const QRect region)
+{
+    m_region = region;
+}
+
 bool ScreenChooserDialog::allowRestore() const
 {
     return m_theDialog->property("allowRestore").toBool();
 }
 
+void ScreenChooserDialog::accept()
+{
+    bool valid = true;
+
+    for (const auto &output : selectedOutputs()) {
+        if (output.outputType() == Output::OutputType::Region) {
+            QScopedPointer<SelectionEditor, QScopedPointerDeleteLater> selectionEditor(new SelectionEditor(this));
+
+            valid = selectionEditor->exec();
+            setRegion(selectionEditor->rect());
+
+            break;
+        }
+    }
+
+    if (!valid) {
+        // if we selected rectangular region, but didn't actually choose a region, start over
+        QTimer::singleShot(0, m_theDialog, SLOT(present()));
+    } else {
+        QuickDialog::accept();
+    }
+}
+
 #include "screenchooserdialog.moc"
