@@ -12,8 +12,8 @@
 #include "waylandintegration.h"
 
 #include <QDateTime>
+#include <QList>
 #include <QMap>
-#include <QVector>
 
 class Screencasting;
 class ScreencastingStream;
@@ -29,7 +29,6 @@ class PlasmaWindow;
 class PlasmaWindowManagement;
 class FakeInput;
 class RemoteBuffer;
-class Output;
 class XdgImporter;
 }
 }
@@ -64,10 +63,10 @@ public:
 
     void acquireStreamingInput(bool acquire);
 
-    Stream startStreamingOutput(quint32 outputName, Screencasting::CursorMode mode);
     Stream startStreamingOutput(QScreen *screen, Screencasting::CursorMode mode);
     Stream startStreamingWindow(const QMap<int, QVariant> &win, Screencasting::CursorMode mode);
     Stream startStreamingWorkspace(Screencasting::CursorMode mode);
+    Stream startStreamingRegion(const QRect region, Screencasting::CursorMode mode);
     Stream startStreamingVirtualOutput(const QString &name, const QSize &size, Screencasting::CursorMode mode);
     void stopStreaming(uint32_t nodeid);
 
@@ -83,14 +82,8 @@ public:
     void requestTouchMotion(quint32 touchPoint, const QPointF &pos);
     void requestTouchUp(quint32 touchPoint);
 
-    QMap<quint32, WaylandOutput> screens();
-
     void setParentWindow(QWindow *window, const QString &parentHandle);
 
-protected Q_SLOTS:
-    void addOutput(quint32 name, quint32 version);
-    void removeOutput(quint32 name);
-
 private:
     Stream startStreaming(ScreencastingStream *stream, const QVariantMap &streamOptions);
     bool eventFilter(QObject *watched, QEvent *event) override;
@@ -98,11 +91,8 @@ private:
     uint m_streamInput = 0;
     bool m_waylandAuthenticationRequested = false;
 
-    quint32 m_output;
     QDateTime m_lastFrameTime;
-    QVector<Stream> m_streams;
-
-    QMap<quint32, WaylandOutput> m_outputMap;
+    QList<Stream> m_streams;
 
     KWayland::Client::FakeInput *m_fakeInput = nullptr;
     Screencasting *m_screencasting = nullptr;
